///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

//
// Copy.cpp : Copy data
//

#include "stdafx.h"
#include "string.h"
#include "ipp.h"

void CompareCopy(int len);

int main(int argc, char* argv[])
{
	int i;

	printf("Length,C,memcpy,ippsCopy\n");
	for (i=4; i< 65537; i*=2) CompareCopy(i);

	return 0;
}

void CompareCopy(int len)
{
	Ipp16s *pSrc = ippsMalloc_16s(len);
	Ipp16s *pDst = ippsMalloc_16s(len);
	int i,j;

	Ipp64u start, end, p1;

	for (i=0; i<len; i++) pSrc[i] = i;
	start = ippCoreGetCpuClocks();
	start = ippCoreGetCpuClocks();

	// Long form:
	//	printf("For length %d...\n", len);
	// Short form:
	printf("%d,",len);

	// Copy using C++
	for (i=0; i<len; i++) pDst[i] = pSrc[i];
	start = ippCoreGetCpuClocks();
	p1 = ippCoreGetCpuClocks();
	for (j=0; j<100; j++)
		for (i=0; i<len; i++) pDst[i] = pSrc[i];
	end = ippCoreGetCpuClocks();

	// Long form:
	//	printf("Clocks using C: %d\n",(Ipp32s)(end - p1 - (p1 - start))/100);
	// Short form:
	printf("%d,",(Ipp32s)(end - p1 - (p1 - start))/100);

	// Copy using memcpy
	memcpy (pDst, pSrc, len*2);
	start = ippCoreGetCpuClocks();
	p1 = ippCoreGetCpuClocks();
	for (j=0; j<100; j++)
		memcpy (pDst, pSrc, len*2);
	end = ippCoreGetCpuClocks();

	// Long form:
	//	printf("Clocks using memcpy: %d\n",
	//	(Ipp32s)(end - p1 - (p1 - start))/100);
	// Short form:
	printf("%d,",(Ipp32s)(end - p1 - (p1 - start))/100);

	// Copy using ippsCopy
	ippsCopy_16s(pSrc, pDst, len);
	start = ippCoreGetCpuClocks();
	p1 = ippCoreGetCpuClocks();
	for (j=0; j<100; j++)
		ippsCopy_16s(pSrc, pDst, len);
	end = ippCoreGetCpuClocks();

	// Long form:
	//	printf("Clocks using ippsCopy: %d\n",
	//	(Ipp32s)(end - p1 - (p1 - start))/100);
	// Short form:
	printf("%d\n",(Ipp32s)(end - p1 - (p1 - start))/100);

	ippsFree(pSrc);
	ippsFree(pDst);
}
